==> fred-hermes-0.5.0.tar.gz: fred-hermes-0.5.0/hermes/message.py <==
"""Common classes and methods for all message types."""
import logging
from abc import ABC, abstractproperty
from datetime import datetime
from typing import Any, Dict, Generic, Sequence, Type, TypeVar, Union
from uuid import UUID

from fred_api.messenger.common_types_pb2 import MessageStatus
from typist import SecretaryClient
from typist.client import TemplateNotFound

LOGGER = logging.getLogger(__name__)
M = TypeVar('M', bound='Message')
E = TypeVar('E', bound='MessageEnvelope')


class Message(ABC):
    """Base class for all message types."""

    def __init__(self, *, type: str = None, secretary_client: SecretaryClient = None):
        self.type = type
        self.secretary_client = secretary_client
        self._secretary_property_cache = {}  # type: Dict[str, str]

    @abstractproperty
    def rendered_fields(self) -> Sequence[str]:
        """List of message fields to be rendered by django-secretary."""

    @classmethod
    def from_dict(cls: Type[M], data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> M:
        """Create message instance from a dictionary.

        Raises:
            ValueError: If required keys for message creation are missing
        """
        try:
            return cls(secretary_client=secretary_client, **data)
        except TypeError:
            raise ValueError('Required element missing from the data argument.')

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with message info.

        This method is useful for message serialization.
        """
        return {
            'type': self.type,
            **(self._get_secretary_client_data() if self.secretary_client else {})
        }

    def _secretary_render(self, name: str, *, template: Union[UUID, str], render_method_name: str = 'render') -> str:
        """Render property by django-secretary method.

        These properties are cached. The template and context attributes are not supposed to be overwritten.
        If you change them, you have to invalidate the cache manually.

        Args:
            name: Name of rendered property. Used for caching the results.
            template: UUID or name of template to be rendered.
            render_method_name: Name of secretary render method.

        Returns:
            Rendered template.

        Raises:
            RuntimeError: If secretary client is not set.
        """
        if name not in self._secretary_property_cache:
            if not self.secretary_client:
                raise RuntimeError("Can't render {}, because secretary_client is not set".format(name))

            self._secretary_property_cache[name] = getattr(self.secretary_client, render_method_name)(
                template, context=getattr(self, 'context', {})
            )
        return self._secretary_property_cache[name]

    def _get_secretary_client_data(self) -> Dict[str, Any]:
        """Return dictionary with SecretaryClient field values and render errors.

        When getting SecretaryClient data, which are being rendered, a TemplateNotFound error can occur. In this case
        None is provided as the value and a record is added into the returned dictionary in the ``render_errors`` list.
        """
        ret = {}  # type: Dict[str, Any]
        errors = []

        for item in self.rendered_fields:
            try:
                ret[item] = getattr(self, item)
            except (RuntimeError, TemplateNotFound, ValueError) as e:
                ret[item] = None
                errors.append('SecretaryClient raised an error in the "{}" property: {}'.format(item, e))
                LOGGER.warning('SecretaryClient raised an error in the "{}" property: {}'.format(item, e))

        if errors:
            ret['render_errors'] = errors

        return ret


class MessageEnvelope(ABC, Generic[M]):
    """Base class for all message envelope types."""

    @abstractproperty
    def message_cls(self) -> Type[M]:
        """Define message type enclosed in the envelope."""

    def __init__(
        self,
        message: M,
        *,
        uid: str = None,
        create_datetime: datetime = None,
        send_datetime: datetime = None,
        delivery_datetime: datetime = None,
        attempts: int = 0,
        status: MessageStatus = None,
        archive: bool = True,
        references: Sequence[Dict[str, Union[UUID, str]]] = None
    ):
        self.uid = uid
        self.create_datetime = create_datetime
        self.send_datetime = send_datetime
        self.delivery_datetime = delivery_datetime
        self.attempts = attempts
        self.status = status
        self.message = message
        self.archive = archive
        self.references = references or []

    @classmethod
    def from_dict(cls: Type[E], data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> E:
        """Create instance from a dictionary.

        Raises:
            ValueError: If required keys for message creation are missing
        """
        message = cls.message_cls.from_dict(  # type: ignore
            data.pop('message_data', {}),
            secretary_client=secretary_client
        )
        return cls(message, **data)

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with message envelope info.

        This method is useful for message envelope serialization.
        """
        return {
            'uid': self.uid,
            'create_datetime': self.create_datetime,
            'send_datetime': self.send_datetime,
            'delivery_datetime': self.delivery_datetime,
            'attempts': self.attempts,
            'status': self.status,
            'message': self.message.to_dict(),
            'archive': self.archive,
            'references': self.references,
        }

==> fred-hermes-1.0.1.tar.gz: fred-hermes-1.0.1/hermes/message.py <==
"""Common classes and methods for all message types."""
import logging
from abc import ABC, abstractproperty
from datetime import datetime
from typing import Any, Dict, Generic, Sequence, Type, TypeVar, Union
from uuid import UUID

from fred_api.messenger.common_types_pb2 import MessageStatus
from typist import SecretaryClient
from typist.client import TemplateNotFound

LOGGER = logging.getLogger(__name__)
M = TypeVar('M', bound='Message')
E = TypeVar('E', bound='MessageEnvelope')


class Message(ABC):
    """Base class for all message types."""

    def __init__(self, *, type: str = None, secretary_client: SecretaryClient = None):
        self.type = type
        self.secretary_client = secretary_client
        self._secretary_property_cache = {}  # type: Dict[str, str]

    @abstractproperty
    def rendered_fields(self) -> Sequence[str]:
        """List of message fields to be rendered by django-secretary."""

    @classmethod
    def from_dict(cls: Type[M], data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> M:
        """Create message instance from a dictionary.

        Raises:
            ValueError: If required keys for message creation are missing
        """
        try:
            return cls(secretary_client=secretary_client, **data)
        except TypeError:
            raise ValueError('Required element missing from the data argument.')

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with message info.

        This method is useful for message serialization.
        """
        return {
            'type': self.type,
            **(self._get_secretary_client_data() if self.secretary_client else {})
        }

    def _secretary_render(self, name: str, *, template: Union[UUID, str], render_method_name: str = 'render') -> str:
        """Render property by django-secretary method.

        These properties are cached. The template and context attributes are not supposed to be overwritten.
        If you change them, you have to invalidate the cache manually.

        Args:
            name: Name of rendered property. Used for caching the results.
            template: UUID or name of template to be rendered.
            render_method_name: Name of secretary render method.

        Returns:
            Rendered template.

        Raises:
            RuntimeError: If secretary client is not set.
        """
        if name not in self._secretary_property_cache:
            if not self.secretary_client:
                raise RuntimeError("Can't render {}, because secretary_client is not set".format(name))

            self._secretary_property_cache[name] = getattr(self.secretary_client, render_method_name)(
                template, context=getattr(self, 'context', {})
            )
        return self._secretary_property_cache[name]

    def _get_secretary_client_data(self) -> Dict[str, Any]:
        """Return dictionary with SecretaryClient field values and render errors.

        When getting SecretaryClient data, which are being rendered, a TemplateNotFound error can occur. In this case
        None is provided as the value and a record is added into the returned dictionary in the ``render_errors`` list.
        """
        ret = {}  # type: Dict[str, Any]
        errors = []

        for item in self.rendered_fields:
            try:
                ret[item] = getattr(self, item)
            except (RuntimeError, TemplateNotFound, ValueError) as e:
                ret[item] = None
                errors.append('SecretaryClient raised an error in the "{}" property: {}'.format(item, e))
                LOGGER.warning('SecretaryClient raised an error in the "{}" property: {}'.format(item, e))

        if errors:
            ret['render_errors'] = errors

        return ret


class MessageEnvelope(ABC, Generic[M]):
    """Base class for all message envelope types."""

    @abstractproperty
    def message_cls(self) -> Type[M]:
        """Define message type enclosed in the envelope."""

    def __init__(
        self,
        message: M,
        *,
        uid: str = None,
        create_datetime: datetime = None,
        send_datetime: datetime = None,
        delivery_datetime: datetime = None,
        attempts: int = 0,
        status: MessageStatus = None,
        archive: bool = True,
        references: Sequence[Dict[str, Union[UUID, str]]] = None
    ):
        self.uid = uid
        self.create_datetime = create_datetime
        self.send_datetime = send_datetime
        self.delivery_datetime = delivery_datetime
        self.attempts = attempts
        self.status = status
        self.message = message
        self.archive = archive
        self.references = references or []

    @classmethod
    def from_dict(cls: Type[E], data: Dict[str, Any], *, secretary_client: SecretaryClient = None) -> E:
        """Create instance from a dictionary.

        Raises:
            ValueError: If required keys for message creation are missing
        """
        message = cls.message_cls.from_dict(  # type: ignore
            data.get('message_data', {}),
            secretary_client=secretary_client
        )
        return cls(message, **{k: v for k, v in data.items() if k != 'message_data'})

    def to_dict(self) -> Dict[str, Any]:
        """Return dictionary with message envelope info.

        This method is useful for message envelope serialization.
        """
        return {
            'uid': self.uid,
            'create_datetime': self.create_datetime,
            'send_datetime': self.send_datetime,
            'delivery_datetime': self.delivery_datetime,
            'attempts': self.attempts,
            'status': self.status,
            'message': self.message.to_dict(),
            'archive': self.archive,
            'references': self.references,
        }
